precision mediump float;
varying vec2 vTextureCoord;
uniform sampler2D uTexture;

uniform float uSaturation;
uniform float uBrightness;


vec3 RGB2HSV(vec3 rgb){

        float R=rgb.x;
        float G=rgb.y;
        float B=rgb.z;

        float h;
        float s;
        float v;
        float max1=max(R,max(G,B));
        float min1=min(R,min(G,B));

        if(max1 != 0.0){
            s=(max1-min1)/max1;
        }else{
            s=0.0;
        }
        if(R==max1){
            h=(G-B)/(max1-min1);
        } else if(G==max1){
            h=2.0+(B-R)/(max1-min1);
        }else if(B==max1){
            h=4.0+(R-G)/(max1-min1);
        }
        h=h*60.0;
        if(h<0.0){
            h=h+360.0;
        }
        v=max1;
        return vec3(h,s,v);

}

//RGB2HSV方法在美颜下红色区域会有密集黑点问题，所以改用此方式
vec3 RGB2HSV_NEW(vec3 rgb){
        vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
            vec4 p = mix(vec4(rgb.bg, K.wz), vec4(rgb.gb, K.xy), step(rgb.b, rgb.g));
            vec4 q = mix(vec4(p.xyw, rgb.r), vec4(rgb.r, p.yzx), step(p.x, rgb.r));
            float d = q.x - min(q.w, q.y);
            float e = 1.0e-10;
            return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 HSV2RGB_NEW(vec3 rgb){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(rgb.xxx + K.xyz) * 6.0 - K.www);
    return rgb.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), rgb.y);
}

vec3 HSV2RGB(vec3 hsv){

        float R;
        float G;
        float B;
        if(hsv.y==0.0){
            R=G=B=hsv.z;
        }else{
            hsv.x=hsv.x/60.0;
//            int i=int(hsv.x);
//            int i=int(hsv.x-6.0*floor(hsv.x/6.0));
            int i=int(floor(hsv.x));
            float f=hsv.x-float(i);
            float a=hsv.z*(1.0-hsv.y);
            float b=hsv.z*(1.0-hsv.y*f);
            float c=hsv.z*(1.0-hsv.y*(1.0-f));


            if(i==0){
                R=hsv.z;
                G=c;
                B=a;
            }else if(i==1){
                R=b;
                G=hsv.z;
                B=a;
            }else if(i==2){
                R=a;
                G=hsv.z;
                B=c;
            }else if(i==3){
                R=a;
                G=b;
                B=hsv.z;
            }else if(i==4){
                R=c;
                G=a;
                B=hsv.z;
            }else if(i==5){
                R=hsv.z;
                G=a;
                B=b;
            }
        }

        return vec3(R,G,B);
}


void main(){
    vec4 textureColor=texture2D(uTexture,vTextureCoord);

    vec3 textureHSV=RGB2HSV_NEW(vec3(textureColor.r,textureColor.g,textureColor.b));
    vec3 resultHSV=vec3(textureHSV.x,textureHSV.y*uSaturation,textureHSV.z*uBrightness);
    gl_FragColor=vec4(HSV2RGB_NEW(resultHSV),1.0);


}